@@ -4,24 +4,22 @@
 #' using Inversion - Best Matching (IBM) approach, see 'Details' below for further information.
 #'
 #' @param samples A list of the K (K>1) samples to be studied, all following admixture distributions.
-#' @param admixMod A list of objects of class 'admix_model', containing useful information about distributions and parameters.
+#' @param admixMod A list of objects of class \link[admix]{admix_model}, containing useful information about distributions and parameters.
 #' @param conf_level (default to 0.95) The confidence level of the k-sample tests used in the clustering procedure.
-#' @param n_sim_tab (default to 100) Number of simulated Gaussian processes when tabulating the inner convergence distribution
-#'                  in the IBM approach.
-#' @param tune_penalty (default to FALSE) A boolean that allows to choose between a classical penalty term or an optimized penalty (embedding
-#'                     some tuning parameters, automatically optimized). Optimized penalty is particularly useful for low or unbalanced sample sizes
-#'                     to detect alternatives to the null hypothesis (H0).
-#' @param parallel (default to FALSE) Boolean to indicate whether parallel computations are performed (speed-up the tabulation).
-#' @param n_cpu (default to 2) Number of cores used when paralleling computations.
+#' @param tune_penalty (default to TRUE) A boolean that allows to choose between a classical penalty term or an optimized penalty (embedding
+#'                     some tuning parameters, automatically optimized). Optimized penalty is particularly useful for low/mid-sized samples,
+#'                     or unbalanced sample sizes to detect alternatives to the null hypothesis (H0). It is recommended to use it.
 #' @param tabul_dist (default to NULL) Only useful for comparisons of detected clusters at different confidence levels. A list of
 #'                    the tabulated distributions of the stochastic integral used in the k-sample test, each element for each
 #'                    cluster previously detected.
-#' @param echo (default to FALSE) Display the remaining computation time.
+#' @param echo (default to TRUE) Display the remaining computation time.
+#' @param ... Optional arguments to \link[admix]{IBM_k_samples_test}; namely 'n_sim_tab', 'parallel' and 'n_cpu'. These are crucial
+#'            to speed-up the building of clusters.
 #'
 #' @references
 #' \insertRef{MilhaudPommeretSalhiVandekerkhove2024b}{admix}
 #'
-#' @return An object of class 'admix_cluster', containing 12 attributes: 1) the number of samples under study; 2) the sizes of samples;
+#' @return An object of class \link[admix]{admix_cluster}, containing 12 attributes: 1) the number of samples under study; 2) the sizes of samples;
 #'         3) the information about mixture components in each sample (distributions and parameters); 4) the number of detected clusters;
 #'         5) the list of p-values for each k-sample test at the origin of detected clusters; 6) the cluster affiliation for each sample;
 #'         7) the confidence level of statistical tests; 8) which samples in which cluster; 9) the size of clusters; 10) the estimated
@@ -64,32 +62,22 @@
 #' ## Clustering procedure:
 #' admix_cluster(samples = list(data1, data2, data3, data4),
 #'               admixMod = list(admixMod1, admixMod2, admixMod3, admixMod4),
-#'               conf_level = 0.95, n_sim_tab = 30, tune_penalty = TRUE,
-#'               parallel = FALSE, n_cpu = 2, tabul_dist = NULL, echo = FALSE)
+#'               conf_level = 0.95, tune_penalty = TRUE, n_sim_tab = 30)
 #' }
 #'
 #' @author Xavier Milhaud <xavier.milhaud.research@gmail.com>
 #' @export
 
-admix_cluster <- function(samples, admixMod, conf_level = 0.95, n_sim_tab = 30, tune_penalty = FALSE,
-                          parallel = FALSE, n_cpu = 2, tabul_dist = NULL, echo = FALSE)
+admix_cluster <- function(samples, admixMod, conf_level = 0.95, tune_penalty = TRUE,
+                          tabul_dist = NULL, echo = TRUE, ...)
 {
   old_options_warn <- base::options()$warn
   base::options(warn = -1)
 
   if (length(sapply(samples, length)) == 1) return("One single sample, no clusters to be found.")
-
-  ## Control whether parallel computations were asked for or not:
-  if (parallel) {
-    `%fun%` <- doRNG::`%dorng%`
-    doParallel::registerDoParallel(cores = n_cpu)
-  } else {
-    `%fun%` <- foreach::`%do%`
-  }
-
   ## Get the minimal size among all sample sizes, useful for future tabulation (adjustment of variance-covariance):
   minimal_size <- min(sapply(X = samples, FUN = length))
-  index_minimal_size <- which.min(sapply(X = samples, FUN = length))
+
   ## K*(K-1)/2 combinations of populations under study:
   couples.list <- NULL
   for (i in 1:(length(samples)-1)) {
@@ -97,9 +85,10 @@ admix_cluster <- function(samples, admixMod, conf_level = 0.95, n_sim_tab = 30,
       couples.list <- rbind(couples.list,c(i,j))
     }
   }
-  couples.expr <- couples.param <- vector(mode = "list", length = nrow(couples.list))
-  empirical.contr <-
-  foreach::foreach (k = 1:nrow(couples.list), .inorder = TRUE, .errorhandling = 'pass', .export = ls(globalenv())) %fun% {
+
+  weights.list <- matrix(data = NA, nrow = nrow(couples.list), ncol = 2)
+  empirical.contr <- vector(mode = "list", length = nrow(couples.list))
+  for (k in 1:nrow(couples.list)) {
     ## Comparison between the two populations :
     XY <- estim_IBM(samples = samples[as.numeric(couples.list[k, ])],
                     admixMod = admixMod[as.numeric(couples.list[k, ])], n.integ = 1000)
@@ -107,10 +96,10 @@ admix_cluster <- function(samples, admixMod, conf_level = 0.95, n_sim_tab = 30,
       XY <- estim_IBM(samples = samples[as.numeric(couples.list[k, ])],
                       admixMod = admixMod[as.numeric(couples.list[k, ])], n.integ = 1000)
     }
-    minimal_size * IBM_empirical_contrast(XY$estimated_mixing_weights, samples = samples[as.numeric(couples.list[k, ])],
-                                          admixMod = admixMod[as.numeric(couples.list[k, ])], G = XY$integ.supp, fixed.p.X = XY$p.X.fixed)
+    weights.list[k, ] <- getmixingWeight(XY)
+    empirical.contr[[k]] <- minimal_size * IBM_empirical_contrast(XY$estimated_mixing_weights, samples = samples[as.numeric(couples.list[k, ])],
+                                                                  admixMod = admixMod[as.numeric(couples.list[k, ])], G = XY$integ.supp, fixed.p.X = XY$p.X.fixed)
   }
-
   ## Manage cases when the optimization algorithm could not find a solution:
   if (length(which(lapply(X = empirical.contr, FUN = is.numeric) == FALSE)) != 0) {
     for (k in 1:length(which(lapply(X = empirical.contr, FUN = is.numeric) == FALSE))) {
@@ -119,173 +108,106 @@ admix_cluster <- function(samples, admixMod, conf_level = 0.95, n_sim_tab = 30,
   }
 
   ## Give a simple and useful representation of results for each couple:
-  H0.test <- contrast.matrix <- discrepancy.id <- discrepancy.rank <- matrix(NA, nrow = length(samples), ncol = length(samples))
-  ## Store results of pairwise tests for equality between unknown components (fasten the future research of potential clusters):
-  weights.list <- matrix(data = NA, nrow = nrow(couples.list), ncol = 2)
+  contrast.matrix <- discrepancy.id <- discrepancy.rank <- matrix(NA, nrow = length(samples), ncol = length(samples))
   for (k in 1:nrow(couples.list)) {
     contrast.matrix[couples.list[k, ][1], couples.list[k, ][2]] <- empirical.contr[[k]]
     discrepancy.id[couples.list[k, ][1], couples.list[k, ][2]] <- paste(couples.list[k, ][1], couples.list[k, ][2], sep = "-")
     discrepancy.rank[couples.list[k, ][1], couples.list[k, ][2]] <- rank(unlist(empirical.contr))[k]
-    ## Pairwise testing: test H0 between the two considered populations.
-    pairwise_H0test <- NULL
-    pairwise_H0test <- IBM_2samples_test(samples = samples[as.numeric(couples.list[k, ])],
-                                         admixMod = admixMod[as.numeric(couples.list[k, ])],
-                                         conf_level = conf_level, parallel = parallel, n_cpu = n_cpu,
-                                         n_sim_tab = n_sim_tab)
-    weights.list[k, ] <- pairwise_H0test$estimated_mixing_weights
-    H0.test[couples.list[k, ][1], couples.list[k, ][2]] <- pairwise_H0test$reject_decision
   }
 
   ## Start algorithm to create the clusters :
   clusters <- tab_distrib <- vector(mode = "list")
-  which_row <- unlist(apply(discrepancy.rank, 2, function(x) which(x == 1)))
-  which_col <- unlist(apply(discrepancy.rank, 1, function(x) which(x == 1)))
-  neighboors_index_init <- which.min(contrast.matrix)
-  first.group <- discrepancy.id[which_row, which_col]
+  closest_couple <- couples.list[which.min(empirical.contr), ]
+  tab_distrib <- NULL
   if (is.null(tabul_dist)) {
-    U <- IBM_tabul_stochasticInteg(samples = list(samples[[which_row]], samples[[which_col]]),
-                                   admixMod = list(admixMod[[which_row]], admixMod[[which_col]]),
-                                   min_size = minimal_size, parallel = parallel, n_cpu = n_cpu,
-                                   n.varCovMat = 80, n_sim_tab = n_sim_tab)
-    Usim <- tab_distrib[[1]] <- U$U_sim
-    CDF_U <- stats::ecdf(U$U_sim)
-    q_H <- stats::quantile(U$U_sim, conf_level)
+    twoSample_test <- IBM_k_samples_test(samples = samples[closest_couple], admixMod = admixMod[closest_couple],
+                                         conf_level = conf_level, sim_U = NULL, tune_penalty = FALSE, ...)
+    tab_distrib <- twoSample_test$tabulated_dist
   } else {
-    Usim <- tabul_dist[[1]]
-    tab_distrib[[1]] <- tabul_dist[[1]]
-    CDF_U <- stats::ecdf(tabul_dist[[1]])
-    q_H <- stats::quantile(tabul_dist[[1]], conf_level)
-  }
-  test.H0 <- contrast.matrix[which_row, which_col] > q_H
-  ## Numerical vector storing the p-values associated to each test: each cluster is closed once the p-value lies below the H0-rejection
-  ## threshold (1-conf_level). This enables to see whether we were close to close the cluster or not each time we add a new population.
-  p_value <- numeric(length = 0L)
-  if (!test.H0) {
-    clusters[[1]] <- alreadyGrouped_samples <- as.character(c(which_row, which_col))
-    neighboors <- c(discrepancy.id[which_row, ], discrepancy.id[which_col, ], discrepancy.id[ ,which_row], discrepancy.id[ ,which_col])
-    new.n_clust <- n_clust <- 1
-    ## Look for a another member to integrate the newly built cluster:
-    indexesSamples_to_consider <- c(0,0,0)
-    p_value <- c(p_value, 1 - CDF_U(contrast.matrix[which_row, which_col]))
-    CDF_U <- NULL
-    if (echo) {
-      prog_bar <- utils::txtProgressBar(min = 0, max = length(samples), style = 3, width = 50, char = "=")
-      utils::setTxtProgressBar(prog_bar, 2/length(samples))
-    }
-  } else {
-    clusters[[1]] <- as.character(which_row)
-    clusters[[2]] <- as.character(which_col)
-    alreadyGrouped_samples <- c(as.character(which_row), as.character(which_col))
-    neighboors <- c(discrepancy.id[which_col, ], discrepancy.id[ ,which_col])
-    n_clust <- 1
-    new.n_clust <- 2
-    ## Look for a second member to integrate the newly second built cluster:
-    indexesSamples_to_consider <- c(0,0)
-    if (echo) {
-      prog_bar <- utils::txtProgressBar(min = 0, max = length(samples), style = 3,  width = 50, char = "=")
-      utils::setTxtProgressBar(prog_bar, 1/length(samples))
-    }
+    twoSample_test <- IBM_k_samples_test(samples = samples[closest_couple], admixMod = admixMod[closest_couple],
+                                         conf_level = conf_level, sim_U = tabul_dist[[1]], tune_penalty = FALSE, ...)
+    tab_distrib <- tabul_dist[[1]]
   }
 
-  while (length(alreadyGrouped_samples) < length(samples)) {
+  Usim <- twoSample_test$tabulated_dist
+  if (!all(is.na(Usim))) {
+    CDF_U <- stats::ecdf(Usim)
+    p_value <- 1 - CDF_U(empirical.contr[[which.min(empirical.contr)]])
+  } else {
+    Usim <- NULL
+    p_value <- 1e-16
+  }
+
+  if (!twoSample_test$reject_decision) {
+    CDF_U <- NULL
+    clusters[[1]] <- as.character(closest_couple)
+    ## Detect which are the neighboors of the current cluster under study:
+    index_neighboors_couples <- c(1:nrow(couples.list))[-c(which.min(empirical.contr),
+                                  which(rowSums(t(apply(couples.list, 1, is.element, set=closest_couple))) == 0))]
+    neighboors_couples <- couples.list[index_neighboors_couples, ]
+    n_clust <- 1
+  } else {
+    clusters[[1]] <- as.character(closest_couple[1])
+    clusters[[2]] <- as.character(closest_couple[2])
     ## Detect which are the neighboors of the current population under study:
-    neighboors_index <- sort(unique(match(x = neighboors[-which(is.na(neighboors))], table = c(discrepancy.id))))
-    ## Remove already studied couples:
-    neighboors_index <- neighboors_index[-which(!is.na(match(x = neighboors_index, table = neighboors_index_init)))]
-    nearest_neighboor_index <- neighboors_index[which.min(contrast.matrix[neighboors_index])]
-    neighboors_index_init <- c(neighboors_index_init, nearest_neighboor_index)
-    first_sample <- strsplit(x = discrepancy.id[nearest_neighboor_index], split="-")[[1]][1]
-    second_sample <- strsplit(x = discrepancy.id[nearest_neighboor_index], split="-")[[1]][2]
-    if (all(c(first_sample,second_sample) %in% alreadyGrouped_samples)) {
-      #print("Already affiliated to one existing cluster")
-      which_row <- which_col <- as.numeric(clusters[[length(clusters)]])
-      neighboors <- c(discrepancy.id[which_row, ], discrepancy.id[ ,which_col])
+    index_neighboors_couples <- c(1:nrow(couples.list))[-c(which.min(empirical.contr),
+                                  which(rowSums(t(apply(couples.list, 1, is.element, set=closest_couple[2]))) == 0))]
+    neighboors_couples <- couples.list[index_neighboors_couples, ]
+    n_clust <- 2
+  }
+
+  if (echo) {
+    prog_bar <- utils::txtProgressBar(min = 0, max = length(samples), style = 3, width = 50, char = "=")
+    utils::setTxtProgressBar(prog_bar, 2/length(samples))
+  }
+  alreadyGrouped_samples <- as.character(closest_couple)
+
+  while (length(alreadyGrouped_samples) < length(samples)) {
+
+    nearest_neighboor_index <- which.min(empirical.contr[index_neighboors_couples])
+    first_sample <- as.character(ifelse(is.null(dim(neighboors_couples)), neighboors_couples[1],
+                                        neighboors_couples[nearest_neighboor_index,1]))
+    second_sample <- as.character(ifelse(is.null(dim(neighboors_couples)), neighboors_couples[2],
+                                         neighboors_couples[nearest_neighboor_index,2]))
+    new_sample <- setdiff(x = c(first_sample,second_sample), y = alreadyGrouped_samples)
+    alreadyGrouped_samples <- c(alreadyGrouped_samples, new_sample)
+
+    ## k-sample test (can be 2-sample test):
+    index_samples <- unique(sort(as.numeric(c(clusters[[length(clusters)]], first_sample, second_sample))))
+    if (is.null(tabul_dist)) {
+      k_sample_test <- IBM_k_samples_test(samples = samples[index_samples], admixMod = admixMod[index_samples],
+                                          conf_level = conf_level, sim_U = Usim, tune_penalty = tune_penalty, ...)
+      tab_distrib <- append(tab_distrib, list(k_sample_test$tabulated_dist))
     } else {
-      indexesSamples_to_consider_new <- sort(as.numeric(unique(c(first_sample, second_sample, clusters[[length(clusters)]]))))
-      ## Tabulate the new inner convergence distribution:
-      if (new.n_clust == (n_clust+1)) {
-        n_clust <- n_clust + 1
-        if ( is.null(tabul_dist) | (new.n_clust > length(tabul_dist)) ) {
-          U <- IBM_tabul_stochasticInteg(samples = list(samples[[as.numeric(first_sample)]], samples[[as.numeric(second_sample)]]),
-                                         admixMod = list(admixMod[[as.numeric(first_sample)]], admixMod[[as.numeric(second_sample)]]),
-                                         min_size = minimal_size, parallel = parallel, n_cpu = n_cpu, n.varCovMat = 80, n_sim_tab = n_sim_tab)
-          ## Store the simulated tabulated distribution:
-          Usim <- U$U_sim
-          tab_distrib <- append(tab_distrib, list(Usim))
-          q_H <- stats::quantile(U$U_sim, conf_level)
-          CDF_U <- stats::ecdf(U$U_sim)
-          p_val <- 1 - CDF_U(contrast.matrix[as.numeric(first_sample), as.numeric(second_sample)])
-        } else {
-          Usim <- tabul_dist[[new.n_clust]]
-          q_H <- stats::quantile(tabul_dist[[new.n_clust]], conf_level)
-          CDF_U <- stats::ecdf(tabul_dist[[new.n_clust]])
-          p_val <- 1 - CDF_U(contrast.matrix[as.numeric(first_sample), as.numeric(second_sample)])
-        }
-        CDF_U <- NULL
-      }
-      if (length(setdiff(indexesSamples_to_consider_new, indexesSamples_to_consider)) > 0) {
-        ## check whether new population could be affected to the existing cluster by looking at results from pairwise equality tests:
-        which_to_test <- indexesSamples_to_consider_new[indexesSamples_to_consider_new %in% as.numeric(clusters[[length(clusters)]]) == FALSE]
-        couples_to_test <- apply(X = t(expand.grid(list(as.numeric(clusters[[length(clusters)]]), which_to_test))), 1, sort)
-        if (!is.matrix(couples_to_test)) {
-          ## case of only one pair of two populations:
-          couples_to_test <- sort(couples_to_test)
-          if (H0.test[couples_to_test[1],couples_to_test[2]] == TRUE) {
-            k_sample_decision <- TRUE
-          } else {
-            k_sample_decision <- FALSE
-            k_sample_pval <- p_val
-          }
-        } else {
-          couples_to_test <- t(apply(X = couples_to_test, 1, sort))
-          ## Case of list of two populations:
-          if (all(H0.test[couples_to_test]) == TRUE) {
-            k_sample_decision <- TRUE
-          } else {
-            ## K-sample test :
-            comp_indices <- sort( c(2*indexesSamples_to_consider_new-1, 2*indexesSamples_to_consider_new) )
-            k_sample_test <- IBM_k_samples_test(samples = samples[indexesSamples_to_consider_new],
-                                                admixMod = admixMod[indexesSamples_to_consider_new],
-                                                conf_level = conf_level, parallel = parallel, n_cpu = n_cpu,
-                                                sim_U = Usim, n_sim_tab = n_sim_tab, tune_penalty = tune_penalty)
-            tab_distrib <- append(tab_distrib, list(k_sample_test$tabulated_dist))
-            k_sample_decision <- k_sample_test$reject_decision
-            k_sample_pval <- k_sample_test$p_value
-          }
-        }
-      }
-      if (!k_sample_decision) {
-        alreadyGrouped_samples <- unique(c(alreadyGrouped_samples, first_sample, second_sample))
-        clusters[[length(clusters)]] <- unique(append(clusters[[length(clusters)]], c(first_sample, second_sample) ))
-        which_row <- which_col <- as.numeric(clusters[[length(clusters)]])
-        neighboors <- c(discrepancy.id[which_row, ], discrepancy.id[ ,which_col])
-        new.n_clust <- n_clust
-        p_value <- c(p_value, k_sample_pval)
-      } else {
-        alreadyGrouped_samples <- unique(c(alreadyGrouped_samples, first_sample, second_sample))
-        clusters <- list(clusters,
-                         as.character(indexesSamples_to_consider_new[which((indexesSamples_to_consider_new %in% as.numeric(clusters[[length(clusters)]])) == F)]))
-        which_row <- which_col <- as.numeric(clusters[[length(clusters)]])
-        neighboors <- c(discrepancy.id[which_row, ], discrepancy.id[ ,which_col])
-        new.n_clust <- n_clust + 1
-        indexesSamples_to_consider_new <- as.numeric(clusters[[length(clusters)]])
-        p_value <- c(p_value, 2e-16)
-      }
+      k_sample_test <- IBM_k_samples_test(samples = samples[index_samples], admixMod = admixMod[index_samples],
+                                          conf_level = conf_level, sim_U = tabul_dist[[n_clust]], tune_penalty = tune_penalty, ...)
+      tab_distrib <- append(tab_distrib, tabul_dist[[n_clust]])
     }
-    indexesSamples_to_consider <- indexesSamples_to_consider_new
+    p_value <- c(p_value, k_sample_test$p_value)
+
+    if (k_sample_test$reject_decision) {
+      n_clust <- n_clust + 1
+      clusters <- append(clusters, new_sample)
+      ## update tabulated distribution to unknown one:
+      Usim <- NULL
+    } else {
+      clusters[[length(clusters)]] <- c(clusters[[length(clusters)]], new_sample)
+    }
+
+    ## define new neighboors to study:
+    index_neighboors_couples <- c(1:nrow(couples.list))[-unique(sort(c(
+      c(1:nrow(couples.list))[-which(lapply(apply(couples.list, 1, setdiff, y = as.numeric(alreadyGrouped_samples)), length) > 0)], # already in existing clusters
+      which(apply(couples.list, 1, setequal, y = as.numeric(c(first_sample, second_sample)))), # last couple studied
+      which(rowSums(t(apply(couples.list, 1, is.element, set = as.numeric(clusters[[length(clusters)]]) ))) == 0) # no link with studied sample
+    )))]
+    neighboors_couples <- couples.list[index_neighboors_couples, ]
+
     if (echo) utils::setTxtProgressBar(prog_bar, length(alreadyGrouped_samples))
+
   } # End of While
 
   if (echo) base::close(prog_bar)
 
-  ## Define function that copies upper triangle to lower triangle to make the matrix become symmetric:
-  f <- function(mat) {
-    mat[lower.tri(mat)] <- t(mat)[lower.tri(mat)]
-    mat
-  }
-  symmetric_dist_mat <- f(contrast.matrix)
-  diag(symmetric_dist_mat) <- 0
-
   n_clust_final <- length(rapply(clusters, function(x) length(x)))
   clusters_affiliation <- matrix(NA, nrow = 2, ncol = length(samples))
   clusters_affiliation[1, ] <- as.numeric(rapply(clusters, function(x) utils::head(x,length(samples))))
@@ -316,9 +238,9 @@ admix_cluster <- function(samples, admixMod, conf_level = 0.95, n_sim_tab = 30,
     clust_pop = clusters_components,
     clust_sizes = sapply(X = clusters_components, length),
     clust_weights = clusters_weights,
-    discrepancy_matrix = symmetric_dist_mat,
-    tab_distributions = unique(tab_distrib)
-    )
+    discrepancy_matrix = contrast.matrix,
+    tab_distributions = lapply(X = unique(tab_distrib[-which(lapply(tab_distrib, length) == 1)]), FUN = sort)
+  )
   class(obj) <- "admix_cluster"
   obj$call <- match.call()
 
@@ -396,3 +318,4 @@ summary.admix_cluster <- function(object, ...)
   cat("\n\nMatrix providing the discrepancies between populations, used in the clustering procedure:\n")
   print(object$discrepancy_matrix)
 }
+
