@@ -7,7 +7,7 @@
 #' The unknown quantities p_i and f_i then have to be estimated.
 #'
 #' @param samples A list of the K (K>0) samples to be studied, all following admixture distributions.
-#' @param admixMod A list of objects of class 'admix_model', containing useful information about distributions and parameters.
+#' @param admixMod A list of objects of class \link[admix]{admix_model}, containing useful information about distributions and parameters.
 #' @param est_method The estimation method to be applied. Can be one of 'BVdk' (Bordes and Vandekerkhove estimator), 'PS' (Patra and Sen
 #'         estimator), or 'IBM' (Inversion Best-Matching approach) in the continuous case (continuous random variable). Only 'IBM' for
 #'         discrete random variables. The same estimation method is performed on each sample if several samples are provided.
@@ -25,7 +25,7 @@
 #' \insertRef{BordesVandekerkhove2010}{admix}
 #' \insertRef{MilhaudPommeretSalhiVandekerkhove2024a}{admix}
 #'
-#' @return An object of class 'admix_estim', containing at least 5 attributes: 1) the number of samples under study; 2) the information
+#' @return An object of class \link[admix]{admix_estim}, containing at least 5 attributes: 1) the number of samples under study; 2) the information
 #'         about the mixture components (distributions and parameters); 3) the sizes of the samples; 4) the chosen estimation technique
 #'         (one of 'BVdk', 'PS' or 'IBM'); 5) the estimated mixing proportions (weights of the unknown component distributions in the
 #'         mixture model). In case of 'BVdk' estimation, one additional attribute corresponding to the estimated location shift parameter
@@ -158,7 +158,7 @@ the two proportions, which should roughly be similar than the true actual ratio.
       }
     }
   }
-  cat("\n")
+  #cat("\n")
   if (x$estimation_method == "Bordes and Vandekerkhove (BVdk)") {
     cat(paste("Estimated location parameters of the unknown component distribution in Sample ", 1:x$n_populations,
               ": ", round(x$estimated_locations,2), sep = ""), sep = "\n")
